﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;
using System.Threading;
using System.Diagnostics;

namespace ConsoleApp7
{
    class Program
    {
        static void Main(string[] args)
        {
            Thread t = new Thread(Method);
            t.Start();

            Console.ReadLine();
        }

        static void Method()
        {
            var sw = new Stopwatch();
            sw.Start();
            var path = AppDomain.CurrentDomain.BaseDirectory;
            var list = new List<Tuple<string, long>>();
            var dir = new DirectoryInfo(path);
            var dirs = dir.GetDirectories();
            foreach (var d in dirs)
            {
                var len = GetDirectoryLength(d.FullName);
                list.Add(new Tuple<string, long>(d.FullName, len));
            }
            list.OrderByDescending(x => x.Item2).ToList().ForEach(x =>
            {
                var l = x.Item2;

                Console.WriteLine($"目录：【{x.Item1}】,大小：【{GetLenStr(x.Item2)}】");
            });
            sw.Stop();
            Console.WriteLine($"耗时：【{(double)sw.ElapsedMilliseconds / 1000}】秒");
        }

        /// <summary>
        /// 获取文件夹大小
        /// </summary>
        /// <param name="dirPath">文件夹路径</param>
        /// <returns></returns>
        static long GetDirectoryLength(string dirPath)
        {
            long temp = 0;
            try
            {
                var dir = new DirectoryInfo(dirPath);
                dir.GetFiles().ToList().ForEach(x => temp += GetFileLength(x.FullName));
                var dirs = dir.GetDirectories();
                if (dirs.Length > 0)
                    dirs.ToList().ForEach(x => temp += GetDirectoryLength(x.FullName));

                return temp;
            }
            catch
            {
                return temp;
            }
        }

        /// <summary>
        /// 获取文件大小
        /// </summary>
        /// <param name="filePath">文件路径</param>
        /// <returns></returns>
        static long GetFileLength(string filePath)
        {
            try
            {
                if (!File.Exists(filePath))
                    return 0;
                var file = new FileInfo(filePath);
                return file.Length;
            }
            catch
            {
                return 0;
            }
        }

        /// <summary>
        /// 单位换算
        /// </summary>
        /// <param name="len">大小（B）</param>
        /// <returns></returns>
        static string GetLenStr(long len)
        {
            if (2014 > len)
                return len + "B";
            if (1048576 > len)
                return (len / 1024) + "KB";
            if (1073741824 > len)
                return (len / 1024 / 1024) + "MB";
            len = len / 1024 / 1024;
            return (len / 1024) + "GB " + (len % 1024) + "MB";
        }
    }
}
